@@ -43,11 +43,11 @@
 
 static void *init(const char *config, unsigned *samplerate)
 {
-	struct rxstate *s = calloc(1, sizeof(struct rxstate));
+	struct rxstate *s;
 	unsigned fec = 3, inlv = 8, bps = 3000;
 	char *cp;
 
-	if (!s)
+	if ((s = calloc(1, sizeof(struct rxstate))) == NULL)
 		errstr(SEV_FATAL, "libdemodnewqpsk: cannot allocate memory");
 	hdlc_initrx(&s->hdlc);
 	cp = strtok(config, ",");
@@ -92,7 +92,7 @@ static int setup(void **state, unsigned samplerate, unsigned *overlap)
 static void demodulate(void **state, int16_t *samples, int numsamp)
 {
 	struct rxstate *s = *state;
-	int i, j;
+	int i;
 #if 0
 	int16_t *diags = alloca(numsamp * sizeof(int16_t));
 	int16_t *dptr = diags;
@@ -100,14 +100,14 @@ static void demodulate(void **state, int16_t *samples, int numsamp)
 
 	for (i = 0; i < numsamp; i++) {
 		s->rxbuf[s->bufptr++] = *samples++ * (1.0 / 32768.0);
-		if (s->bufptr < AliasFilterLen + DecimateRatio * SymbolLen / 2 + s->skip)
+		if (s->bufptr < AliasFilterLen + DecimateRatio + DecimateRatio * SymbolLen / 2 + s->skip)
 			continue;
 		s->skip = 0;
 		newqpskrx(state);
-		s->bufptr -= AliasFilterLen;
-		for (j = 0; j < AliasFilterLen; j++, s->bufptr++)
-			s->rxbuf[j] = s->rxbuf[s->bufptr];
-		s->bufptr = AliasFilterLen;
+		s->bufptr -= AliasFilterLen + DecimateRatio;
+		memmove(s->rxbuf, s->rxbuf + s->bufptr,
+			(AliasFilterLen + DecimateRatio) * sizeof(float));
+		s->bufptr = AliasFilterLen + DecimateRatio;
 	}
 }
 
