@@ -119,6 +119,11 @@ static int modulate_calib(void **state, int16_t *samples, int numsamp)
 		numsamp -= n;
 		cnt += n;
 	}
+	while (numsamp > 0) {
+		*samples++ = 0;
+		numsamp--;
+		cnt++;
+	}
 	s->tuneonly = 0;
 	return cnt;
 }
