@@ -427,7 +427,7 @@ static struct soundmodem_state *init(const char *configfile)
 					     MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (sm == MAP_FAILED)
 		errstr(SEV_FATAL, "Cannot allocate memory for state info");
-	sm->chaccesspar = (struct chaccesspar){ 10, 10, 40, 1, 0 };
+	sm->chaccesspar = (struct chaccesspar){ 0, 10, 64, 0, 0 };
 	if (!(f = fopen(configfile, "r")))
 		errprintf(SEV_FATAL, "Cannot open config file \"%s\"\n", configfile);
 	while (fgets(buf, sizeof(buf), f)) {
@@ -606,11 +606,10 @@ static struct soundmodem_state *init(const char *configfile)
 	apar = log2(sm->samplerateby100 + (sm->samplerateby100 >> 1));
 	if (!(sm->drvmode & SNDDRVMODE_CONVU8))
 		apar++;
-	apar |=  0x00030000; /* 0x7fff0000; */
-	errprintf(SEV_INFO, "audio driver fragment size %08x\n", apar);
+	apar |=  (3 << 16);
 	if (ioctl(sm->audiofd, SNDCTL_DSP_SETFRAGMENT, &apar) == -1)
 		errstr(SEV_ERROR, "ioctl: SNDCTL_DSP_SETFRAGMENT");
-	errprintf(SEV_INFO, "audio driver fragment size %08x\n", apar);
+	errprintf(SEV_INFO, "audio driver: %d fragments of size %d bytes\n", (apar & 0xffff0000) >> 16, 1 << (apar & 0xffff));
 #endif
 	if (ioctl(sm->audiofd, SNDCTL_DSP_NONBLOCK, 0) == -1)
 		errstr(SEV_ERROR, "ioctl: SNDCTL_DSP_NONBLOCK");
@@ -1165,7 +1164,7 @@ static void modem_run(struct soundmodem_state *sm)
 		GETTICK(tick2);
 		sm->modulator[txnr].stat.tx_cycles = tick2 - tick1;
 		if (i <= 0)
-			errprintf(SEV_FATAL, "modulator modulate_calib error\n");
+			errprintf(SEV_FATAL, "modulator modulate_calib error (txdelay - %d)\n", i);
 		transmit(sm, txbuf, i);
 		j -= i;
 	}
@@ -1192,7 +1191,7 @@ static void modem_run(struct soundmodem_state *sm)
 			GETTICK(tick2);
 			sm->modulator[txnr].stat.tx_cycles = tick2 - tick1;
 			if (i <= 0)
-				errprintf(SEV_FATAL, "modulator modulate_calib error\n");
+				errprintf(SEV_FATAL, "modulator modulate_calib error (inter-mode delay - %d)\n", i);
 			transmit(sm, txbuf, i);
 			j -= i;
 		}
@@ -1207,7 +1206,7 @@ static void modem_run(struct soundmodem_state *sm)
 		GETTICK(tick2);
 		sm->modulator[txnr].stat.tx_cycles = tick2 - tick1;
 		if (i <= 0)
-			errprintf(SEV_FATAL, "modulator modulate_calib error\n");
+			errprintf(SEV_FATAL, "modulator modulate_calib error (txtail - %d)\n", i);
 		transmit(sm, txbuf, i);
 		j -= i;
 	}
