@@ -355,13 +355,15 @@ static void rxtune(void **state)
 	/* init deinterleaver */
 	init_inlv(&s->fec);
 	for (i = 0; i < DataCarriers; i++) {
-		s->pherr[i] = 0.0;
+		s->phesum[i] = 0.0;
 		s->pheavg[i] = 0.0;
 		s->dcdavg[i] = 0.0;
-		s->power[i] = 0.0;
+		s->power[i]  = 0.0;
 		s->correl[i] = 0.0;
 		s->fecerrors[i] = 0;
 	}
+
+	s->phemax = 0.0;
 }
 
 /* --------------------------------------------------------------------- */
@@ -370,8 +372,9 @@ static void rxdata(void **state)
 {
 	struct rxstate *s = *state;
 	int rxword[SymbolBits], errword[SymbolBits];
-	int i, j, bits, data, dcd, errs;
+	int i, j, bits, data, dcd, errs, cor1, cor2;
 	unsigned int prev2, prev1, curr;
+	float pherr[DataCarriers];
 	complex z;
 	float x;
 	char buf[256];
@@ -410,7 +413,7 @@ static void rxdata(void **state)
 		bits &= (1 << SymbolBits) - 1;
 
 		/* calculate phase error (`0.5 compensates the bias) */
-		s->pherr[i] = x - (bits + 0.5) * 2 * M_PI / PhaseLevels;
+		pherr[i] = x - (bits + 0.5) * 2 * M_PI / PhaseLevels;
 
 		/* flip the top bit back */
 		bits ^= 1 << (SymbolBits - 1);
@@ -425,35 +428,26 @@ static void rxdata(void **state)
 			if (data & (1 << (SymbolBits - 1 - j)))
 				rxword[j] |= 1 << i;
 
-		/*
-		 * Update DCD, phase error, power and
-		 * sync correlation averages.
-		 */
-		if (!s->updhold) {
-			s->dcdavg[i] = s->pherr[i] * s->pherr[i] * DCDTuneAverWeight + s->dcdavg[i] * (1 - DCDTuneAverWeight);
+		/* skip the rest if still holding updates */
+		if (s->updhold)
+			continue;
 
-			s->pheavg[i] = s->pherr[i] * DCDTuneAverWeight + s->pheavg[i] * (1 - DCDTuneAverWeight);
+		/* update phase error power average */
+		s->dcdavg[i] = avg2(s->dcdavg[i], pherr[i] * pherr[i], DCDTuneAverWeight);
 
-			x = cpwr(s->rxpipe[curr][i]);
-			s->power[i] = avg(s->power[i], x, RxDataSyncFollow);
+		/* update phase error average */
+		s->pheavg[i] = avg2(s->pheavg[i], pherr[i], DCDTuneAverWeight);
 
-			x = ccorI(s->rxpipe[prev1][i], s->rxpipe[curr][i]) - ccorI(s->rxpipe[prev1][i], s->rxpipe[prev2][i]);
-			s->correl[i] = avg(s->correl[i], x, RxDataSyncFollow);
-		}
+		/* update carrier power average */
+		x = cpwr(s->rxpipe[curr][i]);
+		s->power[i] = avg(s->power[i], x, RxDataSyncFollow);
+
+		/* update sync correlation average */
+		x = ccorI(s->rxpipe[prev1][i], s->rxpipe[curr][i]) -
+		    ccorI(s->rxpipe[prev1][i], s->rxpipe[prev2][i]);
+		s->correl[i] = avg(s->correl[i], x, RxDataSyncFollow);
 	}
 
-	/* count carriers that have small enough phase error */
-	dcd = 0;
-	for (i = 0; i < DataCarriers; i++)
-		if (s->dcdavg[i] < DCDThreshold / PhaseLevels / PhaseLevels)
-			dcd++;
-
-	/* decide if this was a "good" symbol */
-	if (dcd < DataCarriers / 2)
-		dcd = 0;
-	else
-		dcd = 1;
-
 	/* feed the data to the decoder */
 	for (i = 0; i < SymbolBits; i++) {
 		rxword[i] = deinlv(&s->fec, rxword[i]);
@@ -461,23 +455,63 @@ static void rxdata(void **state)
 		putbitbatch(state, rxword[i]);
 	}
 
-	/* count FEC errors */
-	if (!s->updhold && dcd) {
+	/* count carriers that have small enough phase error */
+	for (dcd = 0, i = 0; i < DataCarriers; i++)
+		if (s->dcdavg[i] < DCDThreshold)
+			dcd++;
+
+	/* decide if this was a "good" symbol */
+	if (dcd >= DataCarriers / 2) {
+		/* count FEC errors */
 		for (i = 0; i < SymbolBits; i++) {
-			s->bitbatches++;
 			for (j = 0; j < DataCarriers; j++)
 				if (errword[i] & (1 << j))
 					s->fecerrors[j]++;
+			s->bitbatches++;
 		}
+
+		/* sync tracking */
+		for (cor1 = cor2 = i = 0; i < DataCarriers; i++) {
+			if (s->power[i] < fabs(s->correl[i]) * RxSyncCorrThres) {
+				if (s->correl[i] >= 0)
+					cor1++;
+				else
+					cor2++;
+			}
+		}
+
+		if (cor1 > DataCarriers * 2 / 3)
+			s->skip = -DecimateRatio;
+		else if (cor2 > DataCarriers * 2 / 3)
+			s->skip =  DecimateRatio;
+
+		if (s->skip != 0) {
+			for (i = 0; i < DataCarriers; i++)
+				s->correl[i] /= 2.0;
+			errprintf(SEV_INFO, "Correcting sync: %+d\n", s->skip);
+		}
+
+		/* sum up phase errors */
+		for (i = 0; i < DataCarriers; i++)
+			s->phesum[i] += pherr[i] * pherr[i];
+
+		/* sum up maximum possible error */
+		s->phemax += M_PI * M_PI / PhaseLevels / PhaseLevels;
+
+		/* correct frequency error */
+		x = phaseavg(pherr, DataCarriers);
+		s->carrierfreq += RxFreqFollowWeight * x * 2.0 / WindowLen;
+
+		/* increase acceptance */
+		if (s->acceptance < DCDMaxDrop)
+			s->acceptance++;
+	} else {
+		/* drop acceptance */
+		if (s->acceptance > 0)
+			s->acceptance--;
 	}
 
-	/* if the symbol wasn't "good", drop acceptance */
-	if (!dcd)
-		s->acceptance--;
-	else if (s->acceptance < DCDMaxDrop)
-		s->acceptance++;
-
-	if (s->acceptance)
+	if (s->acceptance > 0)
 		return;
 
 	/* DCDMaxDrop subsequent "bad" symbols, it's gone... */
@@ -486,11 +520,16 @@ static void rxdata(void **state)
 	buf[0] = 0;
 	for (i = 0; i < DataCarriers; i++) {
 		errs += s->fecerrors[i];
-		j = strlen(buf);
-		sprintf(buf + j, "%02d ", s->fecerrors[i]);
+		sprintf(buf + strlen(buf), "%02d ", s->fecerrors[i]);
 	}
 	errprintf(SEV_INFO, "FEC errors: %s: %03d / %03d\n", buf, errs, s->bitbatches);
 
+	buf[0] = 0;
+	for (i = 0; i < DataCarriers; i++) {
+		sprintf(buf + strlen(buf), "%2.0f ", 10 * log10(s->phemax * 2 / s->phesum[i]));
+	}
+	errprintf(SEV_INFO, "S/N ratio:  %s dB\n", buf);
+
 	/* go back to idling */
 	init_newqpskrx(state);
 	return;
