@@ -260,8 +260,11 @@ static void encodeword(void **state, int jam)
 	complex z;
 	float phi;
 
-	for (i = 0; i < SymbolBits; i++)
-		s->txword[i] = inlv(&s->fec, s->txword[i]);
+	/* run through interleaver only if not jamming */
+	if (!jam) {
+		for (i = 0; i < SymbolBits; i++)
+			s->txword[i] = inlv(&s->fec, s->txword[i]);
+	}
 
 	j = FirstDataCarr;
 	for (i = 0; i < DataCarriers; i++) {
@@ -282,6 +285,7 @@ static void encodeword(void **state, int jam)
 		/* modulate */
 		phi = data * 2 * M_PI / PhaseLevels;
 
+		/* if jamming, turn by maximum phase error */
 		if (jam)
 			phi += M_PI / PhaseLevels;
 
