@@ -9,9 +9,12 @@
 struct txstate {
 	struct hdlc_transmitter hdlc;
 	struct fecstate fec;
-	unsigned bps, shreg;
+	unsigned bps;
+	unsigned shreg;
 	void (*txroutine) (void **);
-	int statecntr, tuneonly, txdone;
+	int statecntr;
+	int tuneonly;
+	int txdone;
 	float *txwindowfunc;
 	complex tunevect[TuneCarriers];
 	complex datavect[DataCarriers];
